@@ -20,7 +20,7 @@ import hashlib
 import logging
 import urllib.parse
 
-from . import rest_api, APIRequest_pb2 as proto
+from . import rest_api, APIRequest_pb2 as proto, loginv3
 from .subfolder import UserFolderNode, SharedFolderNode, SharedFolderFolderNode, RootFolderNode
 from .record import Record
 from .shared_folder import SharedFolder
@@ -48,7 +48,9 @@ unpad_char = lambda s: s[0:-ord(s[-1])]
 
 def run_command(params, request):
     # type: (KeeperParams, dict) -> dict
-    request['client_version'] = rest_api.CLIENT_VERSION
+    if 'client_version' not in request:
+        request['client_version'] = rest_api.CLIENT_VERSION
+
     return rest_api.v2_execute(params.rest_context, request)
 
 
@@ -72,6 +74,14 @@ install_fido_package_warning = 'You can use Security Key with Commander:\n' +\
 
 def login(params):
     # type: (KeeperParams) -> None
+
+    if params.login_v3:
+        logging.info('Logging in to Keeper Commander')
+        loginv3.LoginV3Flow.login(params)
+        return
+
+    logging.info("Logging in...")
+
     global should_cancel_u2f
     global u2f_response
     global warned_on_fido_package
@@ -87,7 +97,7 @@ def login(params):
             logging.debug('No auth verifier, sending pre-auth request')
             try:
                 pre_login_rs = rest_api.pre_login(params.rest_context, params.user)
-                auth_params = pre_login_rs.salt[0]
+                auth_params = get_correct_salt(pre_login_rs.salt)
                 params.iterations = auth_params.iterations
                 params.salt = auth_params.salt
                 params.auth_verifier = auth_verifier(params.password, params.salt, params.iterations)
@@ -120,6 +130,8 @@ def login(params):
                 expire_days = 0 if expire_token else 9999
                 rq['device_token_expire_days'] = expire_days
 
+        rq['client_version'] = rest_api.LEGACY_CLIENT_VERSION   # this login only supports up to v14 clients.
+
         response_json = run_command(params, rq)
 
         if 'device_token' in response_json:
@@ -147,6 +159,7 @@ def login(params):
         if response_json['result_code'] == 'auth_success' and response_json['result'] == 'success':
             success = True
             logging.debug('Auth Success')
+            store_config = not params.config or params.config.get('user') != params.user
 
             params.session_token = response_json['session_token']
 
@@ -176,7 +189,7 @@ def login(params):
             params.sync_data = True
             params.prepare_commands = True
 
-            if store_config: # save token to config file if the file exists
+            if store_config:    # save token to config file if the file exists
                 params.config['user'] = params.user
 
                 if params.config_filename:
@@ -314,6 +327,11 @@ def accept_account_transfer_consent(params, share_account_to):
     return False
 
 
+def decrypt_aes_plain(data: bytes, key: bytes):
+    cipher = AES.new(key=key, mode=AES.MODE_GCM, nonce=data[:12])
+    return cipher.decrypt_and_verify(data[12:-16], data[-16:])
+
+
 def decrypt_aes(data, key):
     # type: (str, bytes) -> bytes
     decoded_data = base64.urlsafe_b64decode(data + '==')
@@ -328,6 +346,13 @@ def decrypt_data(data, key):
     return unpad_binary(decrypt_aes(data, key))
 
 
+def encrypt_aes_plain(data: bytes, key: bytes):
+    iv = os.urandom(12)
+    cipher = AES.new(key=key, mode=AES.MODE_GCM, nonce=iv)
+    enc_data, tag = cipher.encrypt_and_digest(data)
+    return iv + enc_data + tag
+
+
 def encrypt_aes(data, key):
     iv = os.urandom(16)
     cipher = AES.new(key, AES.MODE_CBC, iv)
@@ -342,6 +367,11 @@ def encrypt_aes_key(key_to_encrypt, encryption_key):
     return (base64.urlsafe_b64encode(encrypted_data).decode()).rstrip('=')
 
 
+def encrypt_rsa_plain(data, rsa_key):
+    cipher = PKCS1_v1_5.new(rsa_key)
+    return cipher.encrypt(data)
+
+
 def encrypt_rsa(data, rsa_public_key):
     cipher = PKCS1_v1_5.new(rsa_public_key)
     encrypted_data = cipher.encrypt(data)
@@ -539,12 +569,13 @@ def sync_down(params):
 
     if 'non_shared_data' in response_json:
         for non_shared_data in response_json['non_shared_data']:
-            try:
-                decrypted_data = decrypt_data(non_shared_data['data'], params.data_key)
-                non_shared_data['data_unencrypted'] = decrypted_data
-                params.non_shared_data_cache[non_shared_data['record_uid']] = non_shared_data
-            except:
-                logging.debug('Non-shared data for record %s could not be decrypted', non_shared_data['record_uid'])
+            if 'data' in non_shared_data and non_shared_data['data']:
+                try:
+                    decrypted_data = decrypt_data(non_shared_data['data'], params.data_key)
+                    non_shared_data['data_unencrypted'] = decrypted_data
+                    params.non_shared_data_cache[non_shared_data['record_uid']] = non_shared_data
+                except:
+                    logging.debug('Non-shared data for record %s could not be decrypted', non_shared_data['record_uid'])
 
     # convert record keys from RSA to AES-256
     if 'record_meta_data' in response_json:
@@ -888,7 +919,7 @@ def create_auth_verifier(password, salt, iterations):
     derived_key = derive_key(password, salt, iterations)
     enc_iter = int.to_bytes(iterations, length=3, byteorder='big', signed=False)
     auth_ver = b'\x01' + enc_iter + salt + derived_key
-    return base64.urlsafe_b64encode(auth_ver).decode().strip('=')
+    return loginv3.CommonHelperMethods.bytes_to_url_safe_str(auth_ver)
 
 
 def create_encryption_params(password, salt, iterations, data_key):
@@ -900,7 +931,7 @@ def create_encryption_params(password, salt, iterations, data_key):
     cipher = AES.new(derived_key, AES.MODE_CBC, enc_iv)
     enc_data_key = cipher.encrypt(data_key + data_key)
     enc_params = b'\x01' + enc_iter + salt + enc_iv + enc_data_key
-    return base64.urlsafe_b64encode(enc_params).decode().strip('=')
+    return loginv3.CommonHelperMethods.bytes_to_url_safe_str(enc_params)
 
 
 def decrypt_encryption_params(encryption_params, password):
@@ -910,8 +941,8 @@ def decrypt_encryption_params(encryption_params, password):
     3 bytes: Iterations, unsigned integer, big endian
     16 bytes: salt
     80 bytes: encrypted data key (broken down further below)
-    16 bytes: IV
-    64 bytes: ciphertextIn
+        16 bytes: IV
+        64 bytes: ciphertextIn
     Key for encrypting the data key: 
         PBKDF2_with_HMAC_SHA256(iterations, salt, master password, 256-bit)
     Encryption method: 256-bit AES, CBC mode, no padding
@@ -951,6 +982,22 @@ def decrypt_encryption_params(encryption_params, password):
     return decrypted_data_key[:32]
 
 
+def decrypt_data_key(params: KeeperParams, encrypted_data_key):
+
+    encrypted_data_key_len = len(encrypted_data_key)
+
+    # [12 bytes: nonce / iv]
+    # [32 bytes: ciphertext]
+    # [16 bytes: auth - tag]
+
+    if encrypted_data_key_len != 60:
+        raise CryptoError('Invalid encryption params: Encrypted data key was unexpected length ' + str(encrypted_data_key_len))
+
+    decryption_key = rest_api.derive_key_v2('data_key', params.password, params.salt, params.iterations)
+
+    return rest_api.decrypt_aes(encrypted_data_key, decryption_key)
+
+
 def get_record(params,record_uid):
     """Return the referenced record cache"""
     record_uid = record_uid.strip()
@@ -1290,7 +1337,6 @@ def communicate_rest(params, request, endpoint):
     raise KeeperApiError('Error', endpoint)
 
 
-
 def communicate(params, request):
     # type: (KeeperParams, dict) -> dict
 
@@ -1558,14 +1604,17 @@ def resolve_record_write_path(params, record_uid):
     # type: (KeeperParams, str) -> dict or None
     return resolve_record_permission_path(params, record_uid, 'can_edit')
 
+
 def resolve_record_share_path(params, record_uid):
     # type: (KeeperParams, str) -> dict or None
     return resolve_record_permission_path(params, record_uid, 'can_share')
 
+
 def resolve_record_view_path(params, record_uid):
     # type: (KeeperParams, str) -> dict or None
     return resolve_record_permission_path(params, record_uid, 'can_view')
 
+
 def resolve_record_access_path(params, record_uid, path=None):
     # type: (KeeperParams, str, dict or None) -> dict
     best_path = None
@@ -1691,7 +1740,7 @@ def query_msp(params):
 
     request = {
         'command': 'get_enterprise_data',
-        'include': ['managed_companies', 'licenses', 'nodes']
+        'include': ['managed_companies']
     }
 
     try:
@@ -1716,21 +1765,10 @@ def query_msp(params):
                                     mc['data'] = json.loads(data.decode('utf-8'))
                                 except Exception as e:
                                     pass
-                    if 'nodes' in response:
-                        for node in response['nodes']:
-                            node['data'] = {}
-                            if 'encrypted_data' in node:
-                                try:
-                                    data = decrypt_data(node['encrypted_data'], tree_key)
-                                    data = fix_data(data)
-                                    node['data'] = json.loads(data.decode('utf-8'))
-                                except Exception as e:
-                                    pass
-
-                    params.enterprise = response
-
+                        if params.enterprise:
+                            params.enterprise['managed_companies'] = response['managed_companies']
     except:
-        params.enterprise = None
+        pass
 
 
 def query_enterprise(params):
@@ -1743,7 +1781,8 @@ def query_enterprise(params):
     request = {
         'command': 'get_enterprise_data',
         'include': ['nodes', 'users', 'teams', 'team_users', 'roles', 'role_enforcements', 'role_privileges',
-                    'role_users', 'managed_nodes', 'role_keys', 'licenses', 'queued_teams', 'queued_team_users']
+                    'role_users', 'managed_nodes', 'role_keys', 'role_keys2', 'licenses', 'queued_teams', 'queued_team_users',
+                    'licenses', 'keys']
     }
     try:
         response = communicate(params, request)
@@ -1801,6 +1840,32 @@ def query_enterprise(params):
         params.enterprise = None
 
 
+def login_and_get_mc_params_login_v3(params: KeeperParams, mc_id):
+
+    resp = loginv3.LoginV3API.loginToMc(params.rest_context, params.session_token, mc_id)
+
+    mc_params = KeeperParams(server=params.server, device_id=params.rest_context.device_id)
+
+    mc_params.config = params.config
+    mc_params.auth_verifier = params.auth_verifier
+    mc_params.mfa_token = params.mfa_token
+    mc_params.salt = params.salt
+    mc_params.iterations = params.iterations
+    mc_params.user = params.user
+    mc_params.password = params.password
+    mc_params.enterprise_id = mc_id
+    mc_params.session_token = params.session_token
+    mc_params.login_v3 = params.login_v3
+    mc_params.data_key = params.data_key
+
+    mc_params.session_token = loginv3.CommonHelperMethods.bytes_to_url_safe_str(resp.encryptedSessionToken)
+    mc_params.msp_tree_key = params.enterprise['unencrypted_tree_key']
+
+    query_enterprise(mc_params)
+
+    return mc_params
+
+
 def login_and_get_mc_params(params, mc_id):
 
     mc_params = KeeperParams(server=params.server, device_id=params.rest_context.device_id)
@@ -1820,3 +1885,11 @@ def login_and_get_mc_params(params, mc_id):
     query_enterprise(mc_params)
 
     return mc_params
+
+
+def get_correct_salt(salts):
+    if len(salts) > 1:
+        salt = next((s for s in salts if s.name.lower() == 'alternate'), salts[0])
+    else:
+        salt = salts[0]
+    return salt
