@@ -25,10 +25,11 @@ from prompt_toolkit import PromptSession
 from prompt_toolkit.shortcuts import CompleteStyle
 from prompt_toolkit.enums import EditingMode
 
-from .commands.msp import get_mc_by_name_or_id, check_int
+from .commands.msp import get_mc_by_name_or_id
+
 from .params import KeeperParams
-from . import display
-from .api import sync_down, login, communicate, query_enterprise, query_msp, login_and_get_mc_params
+from . import display, loginv3, api
+# from .api import sync_down, login, communicate, query_enterprise, query_msp, login_and_get_mc_params, login_and_get_mc_params_login_v3
 from .error import AuthenticationError, CommunicationError, CommandError
 from .subfolder import BaseFolderNode
 from .autocomplete import CommandCompleter
@@ -47,6 +48,7 @@ not_msp_admin_error_msg = 'This command is restricted to Keeper MSP administrato
                           'Company. \nIf you are an MSP administrator then try to run `switch-to-msp` ' \
                           'command before executing this command.'
 
+
 def display_command_help(show_enterprise = False, show_shell = False):
     max_length = functools.reduce(lambda x, y: len(y) if len(y) > x else x, command_info.keys(), 0)
 
@@ -70,9 +72,9 @@ def display_command_help(show_enterprise = False, show_shell = False):
     if show_shell:
         print('  ' + 'shell'.ljust(max_length + 2) + '... ' + 'Use Keeper interactive shell')
 
-    print('  ' + 'c'.ljust(max_length + 2) + '... ' + 'Clear the screen')
-    print('  ' + 'h'.ljust(max_length + 2) + '... ' + 'Show command history')
-    print('  ' + 'q'.ljust(max_length + 2) + '... ' + 'Quit')
+    print('  ' + 'clear|c'.ljust(max_length + 2) + '... ' + 'Clear the screen')
+    print('  ' + 'history|h'.ljust(max_length + 2) + '... ' + 'Show command history')
+    print('  ' + 'quit|q'.ljust(max_length + 2) + '... ' + 'Quit')
 
     print('')
     print('Type \'command -h\' to display help on command')
@@ -113,7 +115,7 @@ def check_if_running_as_mc(params, args):
             raise CommandError('', "You do not have permission to manage company %s. MCs able to manage: %s" % (mc_id, can_manage_mcs))
 
         if mc_id not in mc_params_dict:
-            mc_params = login_and_get_mc_params(params, mc_id)
+            mc_params = api.login_and_get_mc_params(params, mc_id)
             mc_params_dict[mc_id] = mc_params
 
         if msp_params is None:
@@ -127,7 +129,10 @@ def check_if_running_as_mc(params, args):
         # Running commands as Managed Company admin via MSP
 
         if current_mc_id not in mc_params_dict:
-            mc_params = login_and_get_mc_params(params, current_mc_id)
+            if params.login_v3:
+                mc_params = api.login_and_get_mc_params_login_v3(params, current_mc_id)
+            else:
+                mc_params = api.login_and_get_mc_params(params, current_mc_id)
             mc_params_dict[current_mc_id] = mc_params
 
         params = mc_params_dict[current_mc_id]
@@ -155,10 +160,70 @@ def command_and_args_from_cmd(command_line):
 def do_command(params, command_line):
     global current_mc_id
 
-    if command_line == 'h':
-        display.formatted_history(stack)
+    def is_msp(params_local):
+        if params_local.enterprise:
+            if 'licenses' in params_local.enterprise:
+                msp_license = next((x for x in params_local.enterprise['licenses'] if x['lic_status'].startswith('msp')),
+                                   None)
+                if msp_license:
+                    return True
+        return False
 
-    elif command_line == 'c':
+    if command_line.lower() == 'h' or command_line.lower() == 'history':
+        display.formatted_history(stack)
+        return
+
+    if '-h' in command_line.lower():
+        if command_line.lower().startswith('h ') or command_line.lower().startswith('history '):
+            print("usage: history|h [-h]")
+            print("\nShow command history.")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('d ') or command_line.lower().startswith('sync-down '):
+            print("usage: sync-down|d [-h]")
+            print("\nDownload your vault from the Keeper Cloud.")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('c ') or command_line.lower().startswith('cls ') or command_line.lower().startswith('clear '):
+            print("usage: clear|cls|c [-h]")
+            print("\nClear the screen.")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('debug '):
+            print("usage: debug [-h]")
+            print("\nToggle debug mode")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('switch-to-mc '):
+            print("usage: switch-to-mc [-h] mcId")
+            print("\nSwitch user's company to Managed Company.")
+            print("\npositional arguments:")
+            print("  mcId               ID of the Managed Company")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('switch-to-msp '):
+            print("usage: switch-to-msp [-h]")
+            print("\nSwitch user's context back to MSP Company.")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+        elif command_line.lower().startswith('q ') or command_line.lower().startswith('quit '):
+            print("usage: quit|q [-h]")
+            print("\nExit commander")
+            print("\noptional arguments:")
+            print("  -h, --help            show this help message and exit")
+            return
+
+    # Track commands history
+    if len(stack) == 0 or stack[0] != command_line:
+        stack.insert(0, command_line)
+
+    if command_line.lower() == 'c' or command_line.lower() == 'cls' or command_line.lower() == 'clear':
         print(chr(27) + "[2J")
 
     elif command_line == 'debug':
@@ -169,18 +234,18 @@ def do_command(params, command_line):
     elif 'switch-to-mc' in command_line:
 
         if current_mc_id is not None:
-            raise CommandError('switch-to-mc', "Already switch to Managed Company id= %s" % current_mc_id)
+            raise CommandError('switch-to-mc', "Already switched to Managed Company id=%s" % current_mc_id)
 
         cmd, args = command_and_args_from_cmd(command_line)
 
-        if not args or not check_int(args):
+        if not args or not loginv3.CommonHelperMethods.check_int(args):
             raise CommandError('switch-to-mc', "Please provide Managed Company ID as integer. Your input was '%s'" % args)
 
         if not params.enterprise:
             logging.error('This command is restricted to Keeper Enterprise administrators.')
             return
 
-        if 'managed_companies' not in params.enterprise:
+        if not is_msp(params):
             logging.error(not_msp_admin_error_msg)
             return
 
@@ -201,7 +266,7 @@ def do_command(params, command_line):
             logging.error('This command is restricted to Keeper Enterprise administrators.')
             return
 
-        if 'managed_companies' not in params.enterprise:
+        if not is_msp(params):
             logging.error(not_msp_admin_error_msg)
             return
 
@@ -211,8 +276,8 @@ def do_command(params, command_line):
         print("Switching back to MSP")
         current_mc_id = None
 
-        query_enterprise(params)
-        query_msp(params)
+        api.query_enterprise(params)
+        api.query_msp(params)
 
     else:
         cmd, args = command_and_args_from_cmd(command_line)
@@ -243,9 +308,8 @@ def do_command(params, command_line):
                     if not params.session_token:
                         try:
                             prompt_for_credentials(params)
-                            logging.info('Logging in...')
-                            login(params)
-                            sync_down(params)
+                            api.login(params)
+                            api.sync_down(params)
                         except KeyboardInterrupt as e:
                             logging.info('Canceled')
                             return
@@ -257,8 +321,8 @@ def do_command(params, command_line):
                             logging.error('This command is restricted to Keeper Enterprise administrators.')
                             return
 
-                    if cmd in msp_commands and params.enterprise:
-                        if 'managed_companies' not in params.enterprise:
+                    if cmd in msp_commands:
+                        if not is_msp(params):
                             logging.error(not_msp_admin_error_msg)
                             return
 
@@ -271,19 +335,16 @@ def do_command(params, command_line):
                                 'command': 'audit_event_client_logging',
                                 'item_logs': params.event_queue
                             }
-                            communicate(params, rq)
+                            api.communicate(params, rq)
                         except Exception as e:
                             logging.debug('Post client events error: %s', e)
                         params.event_queue.clear()
                     if params.sync_data:
-                        sync_down(params)
+                        api.sync_down(params)
                 return result
             else:
                 display_command_help(show_enterprise=(params.enterprise is not None))
 
-            if len(stack) == 0 or stack[0] != command_line:
-                stack.insert(0, command_line)
-
 
 def runcommands(params):
     keep_running = True
@@ -293,13 +354,13 @@ def runcommands(params):
         for command in params.commands:
             logging.info('Executing [%s]...', command)
             try:
-                if not do_command(params, command):
-                    logging.warning('Command %s failed.', command)
+                do_command(params, command)
             except CommunicationError as e:
                 logging.error("Communication Error: %s", e.message)
             except AuthenticationError as e:
                 logging.error("AuthenticationError Error: %s", e.message)
-            except:
+            except Exception as e:
+                logging.debug(e, exc_info=True)
                 logging.error('An unexpected error occurred: %s', sys.exc_info()[0])
 
         if timedelay == 0:
@@ -312,11 +373,26 @@ def runcommands(params):
                 keep_running = False
 
 
-def prompt_for_credentials(params):
-    while not params.user:
+def prompt_for_username_if_needed(params):
+    if not params.user:
         params.user = getpass.getpass(prompt='User(Email): ', stream=None)
-    while not params.password:
-        params.password = getpass.getpass(prompt='Password: ', stream=None)
+
+        while not params.user:
+            params.user = getpass.getpass(prompt='User(Email): ', stream=None)
+
+
+def prompt_for_credentials(params):
+
+    if not params.login_v3:
+        prompt_for_username_if_needed(params)
+
+        while not params.password:
+            try:
+                params.password = getpass.getpass(prompt='Password: ', stream=None)
+            except KeyboardInterrupt:
+                print('')
+            except EOFError:
+                return 0
 
 
 def force_quit():
@@ -358,22 +434,22 @@ def loop(params):  # type: (KeeperParams) -> int
 
     if params.user:
         if len(params.commands) == 0:
-            if not params.password:
+            if not params.login_v3 and not params.password:
                 logging.info('Enter password for {0}'.format(params.user))
                 try:
-                    params.password = getpass.getpass(prompt='Password: ', stream=None)
+                    if not params.login_v3:
+                        params.password = getpass.getpass(prompt='Password: ', stream=None)
                 except KeyboardInterrupt:
                     print('')
                 except EOFError:
                     return 0
-        if params.password:
-            logging.info('Logging in...')
-            try:
-                login(params)
-                if params.session_token:
-                    do_command(params, 'sync-down')
-            except Exception as e:
-                logging.error(e)
+    # if params.password:
+        try:
+            api.login(params)
+            if params.session_token:
+                do_command(params, 'sync-down')
+        except Exception as e:
+            logging.error(e)
 
     while True:
         command = ''
@@ -403,7 +479,7 @@ def loop(params):  # type: (KeeperParams) -> int
                     if tmer:
                         tmer.cancel()
 
-            if command == 'q':
+            if command.lower() == 'q' or command.lower() == "quit":
                 break
 
             suppress_errno = False
