@@ -17,6 +17,7 @@ import base64
 import json
 import logging
 import requests
+import time
 
 from urllib.parse import urlsplit, urlunsplit
 from email.utils import parseaddr
@@ -25,7 +26,7 @@ from Cryptodome.PublicKey import RSA
 from Cryptodome.Util.asn1 import DerSequence
 from Cryptodome.Math.Numbers import Integer
 
-from .. import api, generator
+from .. import api, generator, loginv3
 from .base import dump_report_data, user_choice
 from .record import RecordAddCommand
 from ..params import KeeperParams
@@ -35,6 +36,7 @@ from ..display import bcolors
 from ..error import KeeperApiError, CommandError
 from .base import raise_parse_exception, suppress_exit, Command
 
+
 EMAIL_PATTERN = r"(?i)^[A-Z0-9._%+-]+@(?:[A-Z0-9-]+\.)+[A-Z]{2,}$"
 
 
@@ -57,7 +59,7 @@ def register_command_info(aliases, command_info):
         command_info[p.prog] = p.description
 
 
-share_record_parser = argparse.ArgumentParser(prog='share-record|sr', description='Change record share permissions')
+share_record_parser = argparse.ArgumentParser(prog='share-record|sr', description='Change the sharing permissions of an individual record')
 share_record_parser.add_argument('-e', '--email', dest='email', action='append', required=True, help='account email')
 share_record_parser.add_argument('-a', '--action', dest='action', choices=['grant', 'revoke', 'owner', 'cancel'],
                                  default='grant', action='store', help='user share action. \'grant\' if omitted')
@@ -67,7 +69,7 @@ share_record_parser.add_argument('record', nargs='?', type=str, action='store',
 share_record_parser.error = raise_parse_exception
 share_record_parser.exit = suppress_exit
 
-share_folder_parser = argparse.ArgumentParser(prog='share-folder|sf', description='Change shared folder permissions')
+share_folder_parser = argparse.ArgumentParser(prog='share-folder|sf', description='Change a shared folders permissions.')
 share_folder_parser.add_argument('-a', '--action', dest='action', choices=['grant', 'revoke'], default='grant',
                                  action='store', help='shared folder action. \'grant\' if omitted')
 share_folder_parser.add_argument('-e', '--email', dest='user', action='append',
@@ -82,11 +84,14 @@ share_folder_parser.add_argument('-s', '--can-share', dest='can_share', action='
                                  help='record permission: can be shared')
 share_folder_parser.add_argument('-d', '--can-edit', dest='can_edit', action='store_true',
                                  help='record permission: can be modified.')
+share_folder_parser.add_argument('-f', '--force', dest='force', action='store_true',
+                                 help='Apply permission changes ignoring default folder permissions. Used on the '
+                                      'initial sharing action')
 share_folder_parser.add_argument('folder', nargs='?', type=str, action='store', help='shared folder path or UID')
 share_folder_parser.error = raise_parse_exception
 share_folder_parser.exit = suppress_exit
 
-share_report_parser = argparse.ArgumentParser(prog='share-report', description='Display report on record sharing')
+share_report_parser = argparse.ArgumentParser(prog='share-report', description='Display report of shared records.')
 share_report_parser.add_argument('--format', dest='format', action='store', choices=['table', 'csv'], default='table',
                                  help='output format.')
 share_report_parser.add_argument('--output', dest='output', action='store',
@@ -95,55 +100,60 @@ share_report_parser.add_argument('-r', '--record', dest='record', action='append
 share_report_parser.add_argument('-e', '--email', dest='user', action='append', help='user email or team name')
 share_report_parser.add_argument('-o', '--owner', dest='owner', action='store_true',
                                  help='record ownership information')
+share_report_parser.add_argument('--share-date', dest='share_date', action='store_true',
+                                 help='include date when the record was shared. This flag will only apply to the '
+                                      'detailed owner report. This data is available only to those users who have '
+                                      'permissions to execute reports for their company. Example of the report that '
+                                      'includes shared date: `share-report -v -o --share-date --format table`')
 share_report_parser.add_argument('-v', '--verbose', dest='verbose', action='store_true',
                                  help='display verbose information')
 share_report_parser.error = raise_parse_exception
 share_report_parser.exit = suppress_exit
 
-record_permission_parser = argparse.ArgumentParser(prog='record-permission', description='Modify record permissions')
+record_permission_parser = argparse.ArgumentParser(prog='record-permission', description='Modify a records permissions.')
 record_permission_parser.add_argument('--dry-run', dest='dry_run', action='store_true',
-                                      help='Display record permissions to be changed')
+                                      help='Display the permissions changes without committing them')
 record_permission_parser.add_argument('--force', dest='force', action='store_true',
-                                      help='Apply permission changes without confirmation')
+                                      help='Apply permission changes without any confirmation')
 record_permission_parser.add_argument('-R', '--recursive', dest='recursive', action='store_true',
-                                      help='Apply permission changes to sub-folders recursively')
+                                      help='Apply permission changes to all sub-folders')
 record_permission_parser.add_argument('--share-record', dest='share_record', action='store_true',
-                                      help='Change direct share record permissions')
+                                      help='Change a records sharing permissions')
 record_permission_parser.add_argument('--share-folder', dest='share_folder', action='store_true',
-                                      help='Change shared folder record permissions')
+                                      help='Change a folders sharing permissions')
 record_permission_parser.add_argument('-a', '--action', dest='action', action='store', choices=['grant', 'revoke'],
-                                      required=True, help='user share action')
+                                      required=True, help='The action being taken')
 record_permission_parser.add_argument('-s', '--can-share', dest='can_share', action='store_true',
-                                      help='record permission: can be shared')
+                                      help='Set record permission: can be shared')
 record_permission_parser.add_argument('-d', '--can-edit', dest='can_edit', action='store_true',
-                                      help='record permission: can be edited')
+                                      help='Set record permission: can be edited')
 record_permission_parser.add_argument('folder', nargs='?', type=str, action='store', help='folder path or folder UID')
 record_permission_parser.error = raise_parse_exception
 record_permission_parser.exit = suppress_exit
 
-register_parser = argparse.ArgumentParser(prog='create-user', description='Create Keeper user')
-register_parser.add_argument('--store-record', dest='store', action='store_true',
-                             help='store credentials into Keeper record (must be logged in)')
-register_parser.add_argument('--generate', dest='generate', action='store_true', help='generate password')
-register_parser.add_argument('--pass', dest='password', action='store', help='user password')
-register_parser.add_argument('--data-center', dest='data_center', choices=['us', 'eu'], action='store',
-                             help='data center.')
-register_parser.add_argument('--node', dest='node', action='store', help='node name or node ID (enterprise only)')
+register_parser = argparse.ArgumentParser(prog='create-user', description='Send an invitation to the user to join Keeper')
+# register_parser.add_argument('--store-record', dest='store', action='store_true',
+#                              help='store credentials into Keeper record (must be logged in)')
+# register_parser.add_argument('--generate', dest='generate', action='store_true', help='generate password')
+# register_parser.add_argument('--pass', dest='password', action='store', help='user password')
+# register_parser.add_argument('--data-center', dest='data_center', choices=['us', 'eu'], action='store',
+#                              help='data center.')
+# register_parser.add_argument('--expire', dest='expire', action='store_true',
+#                              help='expire master password (enterprise only)')
+# register_parser.add_argument('--records', dest='records', action='store',
+#                              help='populate vault with default records (enterprise only)')
+# register_parser.add_argument('--question', dest='question', action='store', help='security question')
+# register_parser.add_argument('--answer', dest='answer', action='store', help='security answer')
 register_parser.add_argument('--name', dest='name', action='store', help='user name (enterprise only)')
-register_parser.add_argument('--expire', dest='expire', action='store_true',
-                             help='expire master password (enterprise only)')
-register_parser.add_argument('--records', dest='records', action='store',
-                             help='populate vault with default records (enterprise only)')
-register_parser.add_argument('--question', dest='question', action='store', help='security question')
-register_parser.add_argument('--answer', dest='answer', action='store', help='security answer')
+register_parser.add_argument('--node', dest='node', action='store', help='node name or node ID (enterprise only)')
 register_parser.add_argument('email', action='store', help='email')
 register_parser.error = raise_parse_exception
 register_parser.exit = suppress_exit
 
 
-file_report_parser = argparse.ArgumentParser(prog='file-report', description='File attachment report')
+file_report_parser = argparse.ArgumentParser(prog='file-report', description='List records with file attachments.')
 file_report_parser.add_argument('-d', '--try-download', dest='try_download', action='store_true',
-                                help='try downloading the attachments')
+                                help='Try downloading every attachment you have access to.')
 file_report_parser.error = raise_parse_exception
 file_report_parser.exit = suppress_exit
 
@@ -160,6 +170,12 @@ class RegisterCommand(Command):
         return 100000
 
     def execute(self, params, **kwargs):
+
+        if params.login_v3:
+            logging.debug("Registering user using Login V3 flow to add users")
+            loginv3.LoginV3API.register_for_login_v3(params, kwargs)
+            return
+
         email = kwargs['email'] if 'email' in kwargs else None
 
         if email:
@@ -178,9 +194,9 @@ class RegisterCommand(Command):
                 logging.warning('User \'%s\' already exists in Keeper', email)
             else:
                 logging.error(rs['message'])
-            return
-
-        password_rules = rs['password_rules']
+            # return
+        else:
+            password_rules = rs['password_rules']
 
         # check enterprise
         verification_code = None
@@ -227,8 +243,8 @@ class RegisterCommand(Command):
                     rs = api.run_command(params, rq)
                     if 'password_rules' in rs:
                         password_rules = rs['password_rules']
-            except:
-                pass
+            except Exception as e:
+                logging.warning(e["message"])
 
         password = kwargs['password'] if 'password' in kwargs else None
         generate = kwargs['generate'] if 'generate' in kwargs else None
@@ -238,14 +254,15 @@ class RegisterCommand(Command):
             while not password:
                 pwd = getpass.getpass(prompt='Password: ', stream=None)
                 failed_rules = []
-                for r in password_rules:
-                    m = re.match(r['pattern'], pwd)
-                    if r['match']:
-                        if m is None:
-                            failed_rules.append(r['description'])
-                    else:
-                        if m is not None:
-                            failed_rules.append(r['description'])
+                if password_rules:
+                    for r in password_rules:
+                        m = re.match(r['pattern'], pwd)
+                        if r['match']:
+                            if m is None:
+                                failed_rules.append(r['description'])
+                        else:
+                            if m is not None:
+                                failed_rules.append(r['description'])
                 if len(failed_rules) == 0:
                     password = pwd
                 else:
@@ -278,21 +295,7 @@ class RegisterCommand(Command):
         enc_salt = os.urandom(16)
         backup_salt = os.urandom(16)
 
-        rsa_key = RSA.generate(2048)
-        private_key = DerSequence([0,
-                                   rsa_key.n,
-                                   rsa_key.e,
-                                   rsa_key.d,
-                                   rsa_key.p,
-                                   rsa_key.q,
-                                   rsa_key.d % (rsa_key.p - 1),
-                                   rsa_key.d % (rsa_key.q - 1),
-                                   Integer(rsa_key.q).inverse(rsa_key.p)
-                                   ]).encode()
-        pub_key = rsa_key.publickey()
-        public_key = DerSequence([pub_key.n,
-                                  pub_key.e
-                                  ]).encode()
+        private_key, public_key = loginv3.CommonHelperMethods.generate_rsa_key_pair()
 
         rq = {
             'command': 'register',
@@ -585,8 +588,12 @@ class ShareFolderCommand(Command):
                         api.load_team_keys(params, [team_uid])
                         team_key = params.key_cache.get(team_uid)
                         if team_key:
-                            to['manage_records'] = sh_fol.get('default_manage_records') or False
-                            to['manage_users'] = sh_fol.get('default_manage_users') or False
+                            if kwargs.get('force'):
+                                to['manage_records'] = mr
+                                to['manage_users'] = mu
+                            else:
+                                to['manage_records'] = sh_fol.get('default_manage_records') or False
+                                to['manage_users'] = sh_fol.get('default_manage_users') or False
                             shared_folder_key = sh_fol['shared_folder_key_unencrypted']
                             if type(team_key) == bytes:
                                 to['shared_folder_key'] = api.encrypt_aes(shared_folder_key, team_key)
@@ -1042,12 +1049,35 @@ class ShareReportCommand(Command):
                 dump_report_data(table, headers, fmt=kwargs.get('format'), filename=kwargs.get('output'))
 
         else:
+
+            include_share_date = kwargs.get('share_date')
             record_owners = {}
             record_shared_with = {}
+            is_an_enterprise_user_by_ref = [True] # To track if use is part of an enterprise. If not then call backend only once.
+            total_record = len(record_uids)
+            count = 0
             for uid in record_uids:
+
+                if include_share_date:
+
+                    rem_count = total_record - count
+                    percent = int(((rem_count/total_record)*100))
+                    percent_indicator_left = int(50 - (percent/2))
+                    percent_indicator_right = int(50 - percent_indicator_left)
+
+                    print(
+                          (bcolors.OKBLUE + 'Generating report: |' + bcolors.ENDC) +
+                          ((bcolors.OKBLUE + '█' + bcolors.ENDC) * percent_indicator_left) +
+                          ((bcolors.OKBLUE + '_' + bcolors.ENDC) * percent_indicator_right) +
+                          (bcolors.OKBLUE + '|' + bcolors.ENDC),
+                          (bcolors.OKBLUE + str(100-percent) + "%" + bcolors.ENDC), end='\x1b[1K\r')
+
+                count = count + 1
+
                 record = params.record_cache[uid]
                 if 'shares' in record:
                     record_shared_with[uid] = []
+                    record_share_details = self.get_record_share_activities(params, uid, is_an_enterprise_user_by_ref) if include_share_date else None
                     if 'user_permissions' in record['shares']:
                         for up in record['shares']['user_permissions']:
                             user_name = up['username']
@@ -1057,7 +1087,8 @@ class ShareReportCommand(Command):
                                 can_edit = up.get('editable') or False
                                 can_share = up.get('shareable') or False
                                 permission = self.get_permission_text(can_edit, can_share)
-                                record_shared_with[uid].append('{0} -> {1}'.format(user_name, permission))
+                                date_shared = self.get_date_for_share(record_share_details, user_name)
+                                record_shared_with[uid].append('{0} -> {1}{2}'.format(user_name, permission, date_shared))
                     if 'shared_folder_permissions' in record['shares']:
                         for sfp in record['shares']['shared_folder_permissions']:
                             shared_folder_uid = sfp['shared_folder_uid']
@@ -1075,12 +1106,14 @@ class ShareReportCommand(Command):
                                     permission = self.get_permission_text(can_edit, can_share)
                                     if 'users' in shared_folder:
                                         for u in shared_folder['users']:
+                                            date_shared = self.get_date_for_share_folder_record(record_share_details, shared_folder_uid)
                                             user_name = u['username']
-                                            record_shared_with[uid].append('{0} => {1}'.format(user_name, permission))
+                                            record_shared_with[uid].append('{0} => {1}{2}'.format(user_name, permission, date_shared))
                                     if 'teams' in shared_folder:
                                         for t in shared_folder['teams']:
-                                            user_name = t['name']
-                                            record_shared_with[uid].append('{0} => {1}'.format(user_name, permission))
+                                            date_shared = self.get_date_for_share_folder_record(record_share_details, shared_folder_uid)
+                                            team_name = t['name']
+                                            record_shared_with[uid].append('{0} => {1}{2}'.format(team_name, permission, date_shared))
 
             if len(record_owners) > 0:
                 headers = ['#', 'Record Title', 'Record UID', 'Owner', 'Shared with']
@@ -1110,6 +1143,136 @@ class ShareReportCommand(Command):
         else:
             return 'View Only' if can_view else 'Launch Only'
 
+    @staticmethod
+    def get_record_share_activities(params: KeeperParams, record_uid: str, is_an_enterprise_user_by_ref):
+
+        if not is_an_enterprise_user_by_ref[0]:
+            # no need to execute the query to get analytics data because we know that user is not part of the enterprise
+            return None
+
+        rq = {
+            'command': 'get_audit_event_reports',
+            'report_type': 'raw',
+            'scope': 'enterprise',
+            'limit': 1000,
+            'order': 'ascending',
+            'filter': {
+                'audit_event_type': [
+                    'share'
+                    , 'record_share_outside_user'
+                    , 'remove_share'
+
+                    , 'folder_add_team'
+                    , 'folder_remove_team'
+
+                    , 'folder_add_record'
+                    , 'folder_remove_record'
+
+                    # ,'change_share'
+                    # ,'transfer_owner'
+                    # ,'accept_share'
+                    # ,'cancel_share'
+                ],
+                'record_uid': record_uid
+            }
+        }
+
+        try:
+            rs = api.communicate(params, rq)
+
+        except Exception as e:
+
+            if e.result_code == 'not_an_enterprise_user':
+                # logging.warning("In order to see shared time details, user must be part of the Keeper account.")
+                is_an_enterprise_user_by_ref[0] = False
+                return None
+            elif e.result_code == 'access_denied':
+                logging.debug("You do not have permissions to access report for your organization. In order to "
+                                "allow user to access reports, ask administrator to grant permission \"Run Reports\" "
+                                "permission.")
+                is_an_enterprise_user_by_ref[0] = False
+                return None
+
+        is_an_enterprise_user_by_ref[0] = True
+
+        rs = api.communicate(params, rq)
+
+        if rs['result'] == 'success':
+            audit_events = rs['audit_event_overview_report_rows']
+
+            # sort list by creation date
+            audit_events = sorted(audit_events, key=lambda i: i['created'])
+
+        else:
+            logging.warning("Error retrieving values")
+
+            return None
+
+        # Remove mutually exclusive records (ex. "share -> remove_share -> share -> remove_share -> share" will
+        # become just "share")
+        latest_record_share_events = {}
+        latest_folder_share_events = {}
+
+        for event in audit_events:
+
+            if 'to_username' in event:  # INDIVIDUAL record sharing
+
+                key = str(event['record_uid']) + '-' + str(event['to_username'])
+
+                if event['audit_event_type'] in ['share', 'record_share_outside_user', 'remove_share']:
+                    if key in latest_record_share_events and event['audit_event_type'] == 'remove_share':
+                        del latest_record_share_events[key]
+                    elif event['audit_event_type'] in ['share', 'record_share_outside_user']:
+                        latest_record_share_events[key] = event
+                    else:
+                        pass
+
+            elif 'shared_folder_uid' in event:  # FOLDER sharing
+
+                key = str(event['record_uid']) + '-' + str(event['shared_folder_uid'])
+
+                if event['audit_event_type'] in ['folder_add_record', 'folder_remove_record']:
+                    if key in latest_folder_share_events and event['audit_event_type'] == 'folder_remove_record':
+                        del latest_folder_share_events[key]
+                    elif event['audit_event_type'] == 'folder_add_record':
+                        latest_folder_share_events[key] = event
+                    else:
+                        pass
+
+        return list(latest_record_share_events.values()) + list(latest_folder_share_events.values())
+
+    @staticmethod
+    def get_date_for_share(share_activity_list: list, user_name: str):
+        if not share_activity_list or len(share_activity_list) == 0:
+            return ""
+
+        activity = next((s for s in share_activity_list if 'to_username' in s and s['to_username'] == user_name), None)
+
+        if activity is None:
+            return ""
+
+        activity_created_ms = activity['created']
+        date_formatted = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(activity_created_ms))
+
+        if activity['audit_event_type'] == 'record_share_outside_user':
+            return '\t(externally shared on {0})'.format(date_formatted)
+        else:
+            return '\t(shared on {0})'.format(date_formatted)
+
+    @staticmethod
+    def get_date_for_share_folder_record(share_activity_list: list, shared_folder_uid: str):
+        if not share_activity_list or len(share_activity_list) == 0:
+            return ""
+
+        activity = next((s for s in share_activity_list if ('folder' in s['audit_event_type'] and s["shared_folder_uid"] == shared_folder_uid)), None)
+
+        if activity is None:
+            return ""
+
+        activity_created_ms = activity['created']
+        date_formatted = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(activity_created_ms))
+
+        return '\t(shared on {0})'.format(date_formatted)
 
 class RecordPermissionCommand(Command):
     def get_parser(self):
