@@ -22,19 +22,22 @@ import tempfile
 import json
 
 from urllib.parse import urlsplit
+
+from google.protobuf.json_format import MessageToDict
 from tabulate import tabulate
 from Cryptodome.Cipher import AES
 from Cryptodome.PublicKey import RSA
 from Cryptodome.Math.Numbers import Integer
 
+from ..display import bcolors
 from ..params import KeeperParams, LAST_RECORD_UID, LAST_FOLDER_UID, LAST_SHARED_FOLDER_UID
 from ..record import Record
-from .. import api
+from .. import api, rest_api, loginv3
 from .base import raise_parse_exception, suppress_exit, user_choice, Command
 from ..subfolder import try_resolve_path, find_folders, get_folder_path
 from . import aliases, commands, enterprise_commands
 from ..error import CommandError
-
+# from ..loginv3 import LoginV3API, CommonHelperMethods
 
 SSH_AGENT_FAILURE = 5
 SSH_AGENT_SUCCESS = 6
@@ -47,6 +50,7 @@ SSH_AGENT_CONSTRAIN_LIFETIME = 1
 
 def register_commands(commands):
     commands['sync-down'] = SyncDownCommand()
+    commands['this-device'] = ThisDeviceCommand()
     commands['delete-all'] = RecordDeleteAllCommand()
     commands['whoami'] = WhoamiCommand()
     commands['login'] = LoginCommand()
@@ -62,25 +66,31 @@ def register_commands(commands):
 def register_command_info(aliases, command_info):
     aliases['d'] = 'sync-down'
     aliases['delete_all'] = 'delete-all'
-    for p in [whoami_parser, login_parser, logout_parser, echo_parser, set_parser, help_parser]:
+    for p in [whoami_parser, this_device_parser, login_parser, logout_parser, echo_parser, set_parser, help_parser]:
         command_info[p.prog] = p.description
     command_info['sync-down|d'] = 'Download & decrypt data'
 
 
-whoami_parser = argparse.ArgumentParser(prog='whoami', description='Information about logged in user')
+whoami_parser = argparse.ArgumentParser(prog='whoami', description='Display information about the currently logged in user.')
 whoami_parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help='verbose output')
 whoami_parser.error = raise_parse_exception
 whoami_parser.exit = suppress_exit
 
+this_device_available_command_verbs = ['rename', 'register', 'persistent-login', 'ip-auto-approve', 'timeout']
+this_device_parser = argparse.ArgumentParser(prog='this-device', description='Display and modify settings of the current device.')
+this_device_parser.add_argument('ops', nargs='*', help="operation str: " + ", ".join(this_device_available_command_verbs))
+this_device_parser.error = raise_parse_exception
+this_device_parser.exit = suppress_exit
 
-login_parser = argparse.ArgumentParser(prog='login', description='Login to Keeper')
+
+login_parser = argparse.ArgumentParser(prog='login', description='Login to Keeper.')
 login_parser.add_argument('-p', '--pass', dest='password', action='store', help='master password')
 login_parser.add_argument('email', nargs='?', type=str, help='account email')
 login_parser.error = raise_parse_exception
 login_parser.exit = suppress_exit
 
 
-logout_parser = argparse.ArgumentParser(prog='logout', description='Logout from Keeper')
+logout_parser = argparse.ArgumentParser(prog='logout', description='Logout from Keeper.')
 logout_parser.error = raise_parse_exception
 logout_parser.exit = suppress_exit
 
@@ -104,20 +114,20 @@ connect_parser.error = raise_parse_exception
 connect_parser.exit = suppress_exit
 
 
-echo_parser = argparse.ArgumentParser(prog='echo', description='Displays argument to output')
+echo_parser = argparse.ArgumentParser(prog='echo', description='Displays an argument to output.')
 echo_parser.add_argument('argument', nargs='?', action='store', type=str, help='argument')
 echo_parser.error = raise_parse_exception
 echo_parser.exit = suppress_exit
 
 
-set_parser = argparse.ArgumentParser(prog='set', description='Set environment variable')
+set_parser = argparse.ArgumentParser(prog='set', description='Set an environment variable.')
 set_parser.add_argument('name', action='store', type=str, help='name')
 set_parser.add_argument('value', action='store', type=str, help='value')
 set_parser.error = raise_parse_exception
 set_parser.exit = suppress_exit
 
 
-help_parser = argparse.ArgumentParser(prog='help', description='Displays help on command')
+help_parser = argparse.ArgumentParser(prog='help', description='Displays help on a specific command.')
 help_parser.add_argument('command', action='store', type=str, help='Commander\'s command')
 help_parser.error = raise_parse_exception
 help_parser.exit = suppress_exit
@@ -150,6 +160,174 @@ class SyncDownCommand(Command):
                 params.sync_data = True
 
 
+class ThisDeviceCommand(Command):
+
+    def get_parser(self):
+        return this_device_parser
+
+    def execute(self, params, **kwargs):
+
+        ops = kwargs.get('ops')
+
+        if len(ops) == 0:
+
+            ThisDeviceCommand.print_device_info(params)
+            return
+
+        if len(ops) >= 1 and ops[0].lower() != 'register':
+            if len(ops) == 1 and ops[0].lower() != 'register':
+                logging.error("Must supply action and value. Available sub-commands: " + ", ".join(this_device_available_command_verbs))
+                return
+
+            if len(ops) != 2:
+                logging.error("Must supply action and value. Available sub-commands: " + ", ".join(this_device_available_command_verbs))
+                return
+
+        action = ops[0].lower()
+
+        def register_device():
+            is_device_registered = loginv3.LoginV3API.register_encrypted_data_key_for_device(params)
+
+            if is_device_registered:
+                print(bcolors.OKGREEN + "Successfully registered device" + bcolors.ENDC)
+            else:
+                print(bcolors.OKGREEN + "Device already registered" + bcolors.ENDC)
+
+        if action == 'rename' or action == 'ren':
+            value = ops[1]
+            loginv3.LoginV3API.rename_device(params, value)
+            print(bcolors.OKGREEN + "Successfully renamed device to '" + value + "'" + bcolors.ENDC)
+
+        elif action == 'register':
+            register_device()
+
+        elif action == 'persistent_login' or action == 'persistent-login' or action == 'pl':
+            value = ops[1]
+
+            value_extracted = ThisDeviceCommand.get_setting_str_to_value('persistent_login', value)
+            loginv3.LoginV3API.set_user_setting(params, 'persistent_login', value_extracted)
+            msg = (bcolors.OKGREEN + "ENABLED" + bcolors.ENDC) if value_extracted == '1' else (bcolors.FAIL + "DISABLED" + bcolors.ENDC)
+            print("Successfully " + msg + " Persistent Login on this device")
+
+            register_device()
+
+            if value_extracted == '1':
+
+                _, this_device = ThisDeviceCommand.get_account_summary_and_this_device(params)
+
+                if this_device:
+                    if 'encryptedDataKeyPresent' not in this_device:
+                        print(bcolors.WARNING + "\tThis device is not registered. To register, run command `this-device register`" + bcolors.ENDC)
+
+        elif action == 'ip_auto_approve' or action == 'ip-auto-approve' or action == 'iaa':
+            value = ops[1]
+
+            value_extracted = ThisDeviceCommand.get_setting_str_to_value('ip_disable_auto_approve', value)
+            loginv3.LoginV3API.set_user_setting(params, 'ip_disable_auto_approve', value_extracted)
+            msg = (bcolors.OKGREEN + "ENABLED" + bcolors.ENDC) if value_extracted == '1' else (bcolors.FAIL + "DISABLED" + bcolors.ENDC)
+            print("Successfully " + msg + " 'ip_auto_approve'")
+
+        elif action == 'timeout' or action == 'to':
+
+            value = ops[1]
+            value_extracted = ThisDeviceCommand.get_setting_str_to_value('logout_timer', value)
+            loginv3.LoginV3API.set_user_setting(params, 'logout_timer', value_extracted)
+            print("Successfully modified 'logout_timer' setting")
+
+        else:
+            raise Exception("Unknown sub-command " + action + ". Available sub-commands: ", ", ".join(this_device_available_command_verbs))
+
+    @staticmethod
+    def get_setting_str_to_value(name: str, value: str):
+
+        name = name.lower()
+        value = value.lower()
+
+        if name == 'persistent_login' or name == 'ip_disable_auto_approve':
+            if value == 'yes' or value == 'y' or value == 'on' or value == '1' or value.lower() == 'true':
+                final_val = '1'
+            elif value == 'no' or value == 'n' or value == 'off' or value == '0' or value.lower() == 'false':
+                final_val = '0'
+            else:
+                raise Exception("Unknown value. Available values 'on', 'off', 'yes', 'y', 'no' or 'n'")
+        elif name == 'logout_timer':
+            if not loginv3.CommonHelperMethods.check_int(value):
+                raise Exception("Entered value is not an integer. Please enter integer")
+            final_val = value
+        else:
+            raise Exception("Unhandled settings name '" + name + "'")
+
+        return final_val
+
+    @staticmethod
+    def get_account_summary_and_this_device(params: KeeperParams):
+        def to_alphanumerics(text):
+            # remove ALL non - alphanumerics
+            return re.sub(r'[\W_]+', '', text)
+
+        def compare_device_tokens(t1: str, t2: str):
+            t1 = to_alphanumerics(t1)
+            t2 = to_alphanumerics(t2)
+
+            return t1 == t2
+
+        acct_summary = loginv3.LoginV3API.accountSummary(params)
+        acct_summary_dict = MessageToDict(acct_summary)
+
+        devices = acct_summary_dict['devices']
+
+        if 'device_token' not in params.config:
+            current_device_token = rest_api.get_device_token(params.rest_context)
+        else:
+            current_device_token = params.config['device_token']
+
+        this_device = next((item for item in devices if compare_device_tokens(item['encryptedDeviceToken'], current_device_token)), None)
+
+        return acct_summary_dict, this_device
+
+    @staticmethod
+    def print_device_info(params: KeeperParams):
+
+        acct_summary_dict, this_device = ThisDeviceCommand.get_account_summary_and_this_device(params)
+
+        print('{:>20}: {}'.format('Device Name', this_device['deviceName']))
+        print("{:>20}: {}".format('Client Version', this_device['clientVersion']))
+
+        if 'encryptedDataKeyPresent' in this_device:
+            print("{:>20}: {}".format('Data Key Present', (bcolors.OKGREEN + 'YES' + bcolors.ENDC) if this_device['encryptedDataKeyPresent'] else (bcolors.FAIL + 'NO' + bcolors.ENDC)))
+        else:
+            print("{:>20}: {}".format('Data Key Present', (bcolors.FAIL + 'missing' + bcolors.ENDC)))
+
+        if 'ipDisableAutoApprove' in acct_summary_dict['settings']:
+            ipDisableAutoApprove = acct_summary_dict['settings']['ipDisableAutoApprove']
+            print("{:>20}: {}".format('IP Auto Approve',
+                                      (bcolors.OKGREEN + 'ON' + bcolors.ENDC)
+                                      if ipDisableAutoApprove else
+                                      (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
+        else:
+            print("{:>20}: {}".format('IP Auto Approve', (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
+
+        if 'persistentLogin' in acct_summary_dict['settings']:
+            persistentLogin = acct_summary_dict['settings']['persistentLogin']
+            print("{:>20}: {}".format('Persistent Login',
+                                      (bcolors.OKGREEN + 'ON' + bcolors.ENDC)
+                                      if persistentLogin else
+                                      (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
+
+        else:
+            print("{:>20}: {}".format('Persistent Login', (bcolors.FAIL + 'OFF' + bcolors.ENDC)))
+
+        if 'logoutTimer' in acct_summary_dict['settings']:
+            logoutTimer = acct_summary_dict['settings']['logoutTimer']
+            logoutTimerMin = int(logoutTimer) / 1000 / 60
+            print("{:>20}: {} minutes".format('Logout Timeout', int(logoutTimerMin)))
+
+        else:
+            print("{:>20}: Default".format('Logout Timeout'))
+
+        print("\nAvailable sub-commands: ", bcolors.OKBLUE + (", ".join(this_device_available_command_verbs)) + bcolors.ENDC)
+
+
 class RecordDeleteAllCommand(Command):
     def execute(self, params, **kwargs):
         uc = user_choice('Are you sure you want to delete all Keeper records on the server?', 'yn', default='n')
@@ -203,17 +381,18 @@ class WhoamiCommand(Command):
             print('{0:>20s}: {1:<20s}'.format('Logged in as', params.user))
             if params.license:
                 print('')
-                account_type = params.license['account_type']
+                account_type = params.license['account_type'] if 'account_type' in params.license else None
                 account_type_name = 'Enterprise' if account_type == 2 \
                     else 'Family Plan' if account_type == 1 \
                     else params.license['product_type_name']
                 print('{0:>20s} {1:>20s}: {2}'.format('Account', 'Type', account_type_name))
                 print('{0:>20s} {1:>20s}: {2}'.format('', 'Renewal Date', params.license['expiration_date']))
                 if 'bytes_total' in params.license:
-                    storage_bytes = params.license['bytes_total']
+                    storage_bytes = int(params.license['bytes_total'])  # note: int64 in protobuf in python produces string as opposed to an int or long.
                     storage_gb = storage_bytes >> 30
+                    storage_bytes_used = params.license['bytes_used'] if 'bytes_used' in params.license else 0
                     print('{0:>20s} {1:>20s}: {2}GB'.format('Storage', 'Capacity', storage_gb))
-                    storage_usage = (params.license['bytes_used'] * 100 // storage_bytes) if storage_bytes != 0 else 0
+                    storage_usage = (int(storage_bytes_used) * 100 // storage_bytes) if storage_bytes != 0 else 0     # note: int64 in protobuf in python produces string  as opposed to an int or long.
                     print('{0:>20s} {1:>20s}: {2}%'.format('', 'Usage', storage_usage))
                     print('{0:>20s} {1:>20s}: {2}'.format('', 'Renewal Date', params.license['storage_expiration_date']))
 
@@ -250,10 +429,10 @@ class LoginCommand(Command):
             if not user:
                 return
 
-            if not password:
+            if not password and not params.login_v3:
                 password = getpass.getpass(prompt='... {0:>16}: '.format('Password'), stream=None).strip()
-            if not password:
-                return
+                if not password:
+                    return
         except KeyboardInterrupt as e:
             logging.info('Canceled')
             return
@@ -261,7 +440,6 @@ class LoginCommand(Command):
         params.user = user.lower()
         params.password = password
 
-        logging.info('Logging in...')
         api.login(params)
 
 
