@@ -5,7 +5,6 @@
 #              |_|            
 #
 # Keeper Commander 
-# Copyright 2015 Keeper Security Inc.
 # Contact: ops@keepersecurity.com
 #
 
@@ -46,6 +45,18 @@ class CommunicationError(Error):
         self.message = message
 
 
+class KeeperApiError(CommunicationError):
+    """Exception raised with failed Keeper API request
+    """
+
+    def __init__(self, result_code, message):
+        CommunicationError.__init__(self, message)
+        self.result_code = result_code
+
+    def __str__(self):
+        return self.message or self.result_code
+
+
 class CryptoError(Error):
     """Exception raised with cryptography issues
 
@@ -55,3 +66,9 @@ class CryptoError(Error):
 
     def __init__(self, message):
         self.message = message
+
+
+class CommandError(Error):
+    def __init__(self, command, message):
+        self.command = command
+        self.message = message
