@@ -12,6 +12,8 @@
 
 import cx_Oracle
 
+# cx_Oracle.init_oracle_client(lib_dir="/Users/[username]]/Downloads/instantclient_19_8") # To initialize
+
 """Commander Plugin for Oracle Database Server
    Dependencies: 
        pip3 install cx_Oracle
@@ -24,29 +26,35 @@ def rotate(record, newpassword):
 
     result = False
 
-    host = record.get('cmdr:host')
-    db = record.get('cmdr:db')
+    host = ""
+
+    if record.get('cmdr:dsn'):
+        dsn_str = record.get('cmdr:dsn')
+    else:
+        host = record.get('cmdr:host')
+        db = record.get('cmdr:db')
+        dsn_str = host + '/' + db
 
     connection = ''
 
     try:
         # Connect to the database
-        connection = cx_Oracle.connect(dsn=host + '/' + db,
+        connection = cx_Oracle.connect(dsn=dsn_str,
                                      user=user,
                                      password=oldpassword)
 
         with connection.cursor() as cursor:
-            print("Connected to %s"%(host))
+            print("Connected to %s" % (dsn_str if record.get('cmdr:dsn') else host))
             # Create a new record
-            sql = 'ALTER USER %s IDENTIFIED BY "%s" ACCOUNT UNLOCK'%(user, newpassword)
+            sql = 'ALTER USER %s IDENTIFIED BY "%s" ACCOUNT UNLOCK' % (user, newpassword)
             cursor.execute(sql)
 
         record.password = newpassword
         result = True
-    except:
-        print("Error during connection to Oracle server")
+    except Exception as e:
+        print("Error during connection to Oracle server: %s", e)
     finally:
         if connection:
             connection.close()
 
-    return result 
\ No newline at end of file
+    return result
\ No newline at end of file
